chiark / gitweb /
add mount enumerator
[elogind.git] / unit.c
diff --git a/unit.c b/unit.c
index c8a6f669615f309506574d21d9c89181b0dc9aba..7723393db95163d112ccc9118a1bb4af64efd14f 100644 (file)
--- a/unit.c
+++ b/unit.c
@@ -155,6 +155,21 @@ int unit_add_name(Unit *u, const char *text) {
         return 0;
 }
 
+int unit_choose_id(Unit *u, const char *name) {
+        char *s;
+
+        assert(u);
+        assert(name);
+
+        /* Selects one of the names of this unit as the id */
+
+        if (!(s = set_get(u->meta.names, (char*) name)))
+                return -ENOENT;
+
+        u->meta.id = s;
+        return 0;
+}
+
 void unit_add_to_load_queue(Unit *u) {
         assert(u);
 
@@ -888,32 +903,41 @@ int set_unit_path(const char *p) {
         return 0;
 }
 
-char *unit_name_escape_path(const char *path, const char *suffix) {
+char *unit_name_escape_path(const char *prefix, const char *path, const char *suffix) {
         char *r, *t;
         const char *f;
-        size_t a, b;
+        size_t a, b, c;
 
         assert(path);
-        assert(suffix);
 
/* Takes a path and a util suffix and makes a nice unit name
* of it, escaping all weird chars on the way.
          *
-         * / becomes _, and all chars not alloweed in a unit name get
-         * escaped as \xFF, including the _ and the \ itself, of
-         * course. This escaping is hence reversible.
+         * / becomes ., and all chars not alloweed in a unit name get
+         * escaped as \xFF, including \ and ., of course. This
+         * escaping is hence reversible.
          */
 
-        a = strlen(path);
-        b = strlen(suffix);
+        if (!prefix)
+                prefix = "";
+
+        if (!suffix)
+                suffix = "";
 
-        if (!(r = new(char, a*4+b+1)))
+        a = strlen(prefix);
+        b = strlen(path);
+        c = strlen(suffix);
+
+        if (!(r = new(char, a+b*4+c+1)))
                 return NULL;
 
-        for (f = path, t = r; *f; f++) {
+        memcpy(r, prefix, a);
+
+        for (f = path, t = r+a; *f; f++) {
                 if (*f == '/')
-                        *(t++) = '_';
-                else if (*f == '_' || *f == '\\' || !strchr(VALID_CHARS, *f)) {
+                        *(t++) = '.';
+                else if (*f == '.' || *f == '\\' || !strchr(VALID_CHARS, *f)) {
                         *(t++) = '\\';
                         *(t++) = 'x';
                         *(t++) = hexchar(*f > 4);
@@ -922,7 +946,7 @@ char *unit_name_escape_path(const char *path, const char *suffix) {
                         *(t++) = *f;
         }
 
-        memcpy(t, suffix, b+1);
+        memcpy(t, suffix, c+1);
 
         return r;
 }
@@ -888,32 +903,41 @@ int set_unit_path(const char *p) {
 
-        /* Takes a path and a util suffix and makes a nice unit name
-         * of it, escaping all weird chars on the way.
+        /* Takes a path and a suffix and prefix and makes a nice
+         * string suitable as unit name of it, escaping all weird
+         * chars on the way.
          *